function tracedespike,image,$
  loud=loud,   $             ; if set, print some diagnostics
  threshold=threshold,$      ; fractional threshold for brightness
  ri=ri,$                    ; replacement index: ri=7 or 8 for median filter
  minimum=minimum,$          ; do not correct pixels dimmer than minimum
  statistics=statistics,$    ; use statistical correction beyond stat.. sigma
  ampl=ampl                  ; amplification of CCD signal
;
;+
;   Name: tracedespike
;
;   Purpose: obvious
;
;   Input Parameters:
;      image - trace image to despike  

;   Calling Examples:
;  imout=trace_despike(trace_despike(image))   for double correction on faint fields
;  imout=trace_despike(image,statistics=6)     correct > 6sigma outliers
;  imout=trace_despike(image,minimum=100,ri=3) for low-quartile corr. above I=100
;
;   Restrictions:
;      Note that there are two mutually exlusive keyword sets:
;      for criterion of relative intensity: threshold, minimum
;      for statistical correction:          statistics, ampl
;
;   Notes:
;   A 1024 square image takes about 2 seconds to correct, even for ~10000 hits
;   Time scales roughly linearly with the size of the array
;
;   History:
;      20-May-1998   Karel Schrijver
;       5-June-1998  S.L. Freeland - added /LOUD, made quiet the default
;      30-June-1998  Zarro (SAC/GSFC) - added check for divides by zero
;       
;-
version=1.0
  loud=keyword_set(loud)
  ;replacement threshold
if keyword_set(threshold) then threshold=threshold else threshold=0.15
  ;replacement index
if keyword_set(ri) then ri=ri else ri=7 ; close to median
  ;minimal required brightness
if keyword_set(minimum) then minimum=minimum else minimum=140  
  ; number of standard deviations
if keyword_set(statistics) then sigma=statistics else sigma=6.
  ; electronic amplification
if keyword_set(ampl) then ampl=ampl else ampl=14.
  ; determine array dimensions, and expand the array with two rows and columns
  ; on all sides, and determine the 3x3 boxcar average
d=size(image) 
nx=d(1) 
ny=d(2) 
imout=replicate(1,nx+4,ny+4)
imout(2,2)=image 
imring=smooth(imout,3)       ; slower: ((smooth(imout,3)*9)<32000-imout)/8
  ; candidate replacement map
map=replicate(0.,nx+4,ny+4)
  ; find both exceptionally high and low values
if keyword_set(statistics) then begin
  if loud then print,'Eliminating outliers beyond ',fix(sigma+.5),$
    ' sigma using A=',ampl
  map(3,3)=((imout-imring))(3:nx,3:ny) 
  imap=where(map^2*(ampl/sigma^2) gt imout)
endif else begin
if loud then print,'Eliminating positive outliers deviating more than',fix(threshold*100),$ 
    ' percent.'

;-- check for divides by zero (DMZ)

  bad=where(imout eq 0.,cbad)
  denom=float(imout)
  if cbad gt 0 then denom(bad)=1.
  rat=(imout-imring)/denom
  if cbad gt 0 then rat(bad)=0.
  map(3,3)=rat(3:nx,3:ny) 
  imap=where(map gt threshold and imout gt minimum)
endelse
  ; ring array 5x5
five=[-2*nx-2,-2*nx-1,-2*nx,-2*nx+1,-2*nx+2,-nx-2,-nx+2,-2,2,nx-2,nx+2,$
       2*nx-2,2*nx-1,2*nx,2*nx+1,2*nx+2]
  ; replace hits by chosen value
imout(2,2)=image 
if imap(0) ge 0 then for i=0l,n_elements(imap)-1 do $
  imout(imap(i))=imout(imap(i)+five((sort(imout(imap(i)+five)))(ri))) 
  ; cutout the array and return
if loud then print,'Corrected ',n_elements(imap),' radiation hits and bad pixels'

return,imout(2:1+nx,2:1+ny)
end
